package yrpg.bamp.basedata.plugin.operate.impl;

import kd.bos.algo.DataSet;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.db.DB;
import kd.bos.db.DBRoute;
import kd.bos.db.SqlBuilder;
import kd.bos.db.tx.TX;
import kd.bos.db.tx.TXHandle;
import kd.bos.entity.MainEntityType;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.MetadataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.threads.ThreadPool;
import kd.bos.threads.ThreadPools;
import yrpg.bamp.basedata.common.BasicDataInsertEntity;
import yrpg.bamp.basedata.plugin.operate.BasicDataInsert;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;


public class BasicDataInsertServiceImpl implements BasicDataInsert {
    private static final Log log = LogFactory.getLog(BasicDataInsertServiceImpl.class);
    /**
     * 医院-院区分录标识
     */
    private static String[] CAMPUS_KEY = new String[]{"branchesid", "district_address", "district", "area", "longitude", "latitude", "master_number", "area_code", "is_active"};


    /**
     * 需更新单据标识：
     * 终端表(正式)
     * 医院
     * 辖区管理
     * 医生
     */
    @Override
    public void dataInsert(BasicDataInsertEntity basicEntity) {
        String tag = basicEntity.getLocalName() + "需更新表数据处理" + UUID.randomUUID().toString().replace("-", "");


        String localName = basicEntity.getLocalName();
        String synKey = basicEntity.getSynKey();
        String sourceKey = basicEntity.getSourceKey();
        String statusKey = basicEntity.getStatusKey();
        if (StringUtils.equals("unknow", synKey)) {
            throw new RuntimeException(localName + "需更新表数据处理失败：未获取到正确的结果表标识。");
        }
        //同步所有字段
        Set<String> fieldset = getFieldKey(basicEntity.getSynKey());
        if (localName.equals("医院")) {
            fieldset.removeIf(ox -> StringUtils.equals(ox, statusKey) || StringUtils.equals(ox, "yrpg_sourcebillid")
                    || StringUtils.equals(ox, "yrpg_branchesid") || StringUtils.equals(ox, "yrpg_district_address")
                    || StringUtils.equals(ox, "yrpg_district") || StringUtils.equals(ox, "yrpg_area")
                    || StringUtils.equals(ox, "yrpg_longitude") || StringUtils.equals(ox, "yrpg_latitude")
                    || StringUtils.equals(ox, "yrpg_master_number") || StringUtils.equals(ox, "yrpg_area_code")
                    || StringUtils.equals(ox, "yrpg_is_active") || StringUtils.equals(ox, "yrpg_soumodifytime")

            );
        } else if (localName.equals("客户管理")) {
            fieldset.removeIf(ox -> StringUtils.equals(ox, statusKey) || StringUtils.equals(ox, "yrpg_sourcebillid")
                    || StringUtils.equals(ox, "yrpg_src_sys_code") || StringUtils.equals(ox, "yrpg_address")
                    || StringUtils.equals(ox, "yrpg_default_address") || StringUtils.equals(ox, "yrpg_link_man")
                    || StringUtils.equals(ox, "yrpg_contact_address") || StringUtils.equals(ox, "yrpg_contact_phone")
                    || StringUtils.equals(ox, "yrpg_contact_telephone") || StringUtils.equals(ox, "yrpg_contact_email")
                    || StringUtils.equals(ox, "yrpg_orgid") || StringUtils.equals(ox, "yrpg_office_id")
                    || StringUtils.equals(ox, "yrpg_office_name") || StringUtils.equals(ox, "yrpg_sales_id")
                    || StringUtils.equals(ox, "yrpg_goods_group_id") || StringUtils.equals(ox, "yrpg_start_date")
                    || StringUtils.equals(ox, "yrpg_end_date") || StringUtils.equals(ox, "yrpg_goods_id")
                    || StringUtils.equals(ox, "yrpg_goods_name") || StringUtils.equals(ox, "yrpg_soumodifytime")
            );
        } else if (localName.equals("辖区管理")) {
            fieldset.removeIf(ox -> StringUtils.equals(ox, statusKey) || StringUtils.equals(ox, "yrpg_sourcebillid")
                    || StringUtils.equals(ox, "modifierfield") || StringUtils.equals(ox, "modifydatefield") || StringUtils.equals(ox, "yrpg_soumodifytime")
            );
        } else if (localName.equals("人员档案")) {
            fieldset.removeIf(ox -> StringUtils.equals(ox, statusKey) || StringUtils.equals(ox, "yrpg_sourcebillid")
                    || StringUtils.equals(ox, "yrpg_firstsalesteam") || StringUtils.equals(ox, "yrpg_secondsalesteam")
                    || StringUtils.equals(ox, "yrpg_auditor") || StringUtils.equals(ox, "yrpg_auditdate")
                    || StringUtils.equals(ox, "yrpg_sourcefid") || StringUtils.equals(ox, "yrpg_soumodifytime")
            );
        } else {
            fieldset.removeIf(ox -> StringUtils.equals(ox, statusKey) || StringUtils.equals(ox, "yrpg_sourcebillid")
                    || StringUtils.equals(ox, "yrpg_soumodifytime"));

        }
        Set<String> sourceAllKey = getFieldKey(basicEntity.getSourceKey());

        DynamicObjectCollection queryResult = QueryServiceHelper.query(sourceKey, "id", null
//                new QFilter[]{new QFilter(basicEntity.getBillNoKey(), QCP.equals, "YZJH00077676")}
        );
        Object[] ids = queryResult.stream().map(e -> e.getLong("id")).toArray();


        ThreadPool threadPool = ThreadPools.newCachedThreadPool(tag, 0, 10);
        int totalSize = ids.length;
        int batachSize = 2000;
        for (int i = 0; i * batachSize < totalSize; i++) {
            int fromIndex = i * batachSize;
            int currentIndex = Math.min((i + 1) * batachSize, totalSize);
            List<Object> idList = java.util.Arrays.asList(ids).subList(fromIndex, currentIndex);
            threadPool.submit(() -> {
                String threadTag = tag + "-" + currentIndex;

                if (!idList.isEmpty()) {
                    TXHandle h = TX.required(threadTag);
                    try {
                        //数据处理
                        //-------------
                        // -------------
                        DynamicObject[] subList = BusinessDataServiceHelper.load(idList.toArray(), MetadataServiceHelper.getDataEntityType(sourceKey));
                        List<Object> billId = dataProcessing(subList, basicEntity, fieldset, sourceAllKey);
                        if (billId.size() > 0 && localName.equals("人员档案")) {
                            //UPDATE tk_yrpg_mkt_user SET fid= fk_yrpg_sourcefid where fk_yrpg_sourcebillid in ?
                            SqlBuilder sqlBuilder = new SqlBuilder();
                            sqlBuilder.append("UPDATE tk_yrpg_mkt_user SET fid = fk_yrpg_sourcefid where");
                            sqlBuilder.appendIn("fk_yrpg_sourcebillid", billId);
                            boolean execute = DB.execute(DBRoute.basedata, sqlBuilder);
                            System.out.println(execute);
                        }
                    } catch (Throwable e) {
                        log.error(e.getMessage());
                        h.markRollback();
                    } finally {
                        h.close();
                    }
                }
                return null;
            });
        }
    }


    private List<Object> dataProcessing(DynamicObject[] subList, BasicDataInsertEntity basicEntity, Set<String> fieldset, Set<String> sourceAllKey) {
        String localName = basicEntity.getLocalName();
        String statusKey = basicEntity.getStatusKey();
        String billNoKey = basicEntity.getBillNoKey();
        List<Object> sourceBillId = new ArrayList<>();

        Map<String, String> billKey = getBillKey(localName);
        List<String> largeTextKey = getLargeTextKey(localName);

        DynamicObject basicData = null;
        for (DynamicObject data : subList) {
            String billNo = data.getString(billNoKey);
            //查询基础服务云，需同步单据
            QFilter filter = new QFilter(billNoKey, QCP.equals, billNo);
            if (localName.equals("人员档案")) {
                filter.and("yrpg_sourcebillid", QCP.equals, data.getString("id"));
            }
            DynamicObject basicDataResult = null;
            DynamicObject[] dataResult = BusinessDataServiceHelper.load(basicEntity.getSynKey(), "id", filter.toArray());
           if (dataResult.length == 1) {
                basicDataResult = dataResult[0];
            } else if (dataResult.length > 1) {
                continue;
            }
            if (null != basicDataResult) {
                //修改
                basicData = BusinessDataServiceHelper.loadSingle(basicDataResult.getPkValue(), basicEntity.getSynKey());
                for (String mapKey : fieldset) {
                    String filedKey;
                    if (mapKey.contains("yrpg_")) {
                        int index = mapKey.indexOf("yrpg_") + 5;
                        filedKey = mapKey.substring(index);
                    } else {
                        filedKey = mapKey;
                    }
                    if (billKey.containsKey(filedKey)) {
                        filedKey = billKey.get(filedKey);
                    }
                    if (sourceAllKey.contains(filedKey)) {
                        if (localName.equals("医院") && !Arrays.asList(CAMPUS_KEY).contains(filedKey)) {
                            basicData.set(mapKey, data.get(filedKey));
                            if (largeTextKey.contains(mapKey)) {
                                basicData.set(mapKey + "_tag", data.get(filedKey + "_tag"));
                            }
                        } else {
                            basicData.set(mapKey, data.get(filedKey));
                            if (largeTextKey.contains(mapKey)) {
                                basicData.set(mapKey + "_tag", data.get(filedKey + "_tag"));
                            }
                        }
                    }
                }
                if (localName.equals("医院")) {
                    DynamicObjectCollection basicHospital = basicData.getDynamicObjectCollection("yrpg_rm_med_trd_hospital");
                    DynamicObjectCollection hospitalEntry = data.getDynamicObjectCollection("rm_med_trd_hospital_detai");
                    for (DynamicObject row : hospitalEntry) {
                        String branchesId = row.getString("branchesid");
                        List<DynamicObject> collect = basicHospital.stream().filter(ox -> ox.getString("yrpg_branchesid").equals(branchesId)).collect(Collectors.toList());
                        if (collect.size() > 0) {
                            for (DynamicObject rowList : collect) {
                                rowList.set("yrpg_branchesid", row.get("branchesid"));
                                rowList.set("yrpg_district_address", row.get("district_address"));
                                rowList.set("yrpg_district_address_tag", row.get("district_address_tag"));
                                rowList.set("yrpg_district", row.get("district"));
                                rowList.set("yrpg_area", row.get("area"));
                                rowList.set("yrpg_longitude", row.get("longitude"));
                                rowList.set("yrpg_latitude", row.get("latitude"));
                                rowList.set("yrpg_master_number", row.get("master_number"));
                                rowList.set("yrpg_area_code", row.get("area_code"));
                                rowList.set("yrpg_is_active", row.get("is_active"));
                            }
                        } else {
                            DynamicObject newEntryRow = basicHospital.addNew();
                            newEntryRow.set("yrpg_branchesid", row.get("branchesid"));
                            newEntryRow.set("yrpg_district_address", row.get("district_address"));
                            newEntryRow.set("yrpg_district_address_tag", row.get("district_address_tag"));
                            newEntryRow.set("yrpg_district", row.get("district"));
                            newEntryRow.set("yrpg_area", row.get("area"));
                            newEntryRow.set("yrpg_longitude", row.get("longitude"));
                            newEntryRow.set("yrpg_latitude", row.get("latitude"));
                            newEntryRow.set("yrpg_master_number", row.get("master_number"));
                            newEntryRow.set("yrpg_area_code", row.get("area_code"));
                            newEntryRow.set("yrpg_is_active", row.get("is_active"));
                        }
                    }
                } else if (localName.equals("医生")) {
                    basicData.set("yrpg_datasource", "B");
                } else if (localName.equals("客户管理")) {
                    //来源系统
                    DynamicObjectCollection oldSrcSys = data.getDynamicObjectCollection("customer_src_sys");
                    DynamicObjectCollection newSrcSys = basicData.getDynamicObjectCollection("yrpg_customer_src_sys");
                    newSrcSys.clear();
                    for (DynamicObject row : oldSrcSys) {
                        DynamicObject newEntryRow = newSrcSys.addNew();
                        newEntryRow.set("yrpg_src_sys_code", row.get("src_sys_code"));
                    }
                    //客户所在地址
                    DynamicObjectCollection oldAddress = data.getDynamicObjectCollection("customer_address");
                    DynamicObjectCollection newAddress = basicData.getDynamicObjectCollection("yrpg_customer_address");
                    newAddress.clear();
                    for (DynamicObject row : oldAddress) {
                        DynamicObject newEntryRow = newAddress.addNew();
                        newEntryRow.set("yrpg_address", row.get("address"));
                        newEntryRow.set("yrpg_default_address", row.get("default_address"));
                    }
                    //联系方式
                    DynamicObjectCollection oldManner = data.getDynamicObjectCollection("contact_manner");
                    DynamicObjectCollection newManner = basicData.getDynamicObjectCollection("yrpg_contact_manner");
                    newManner.clear();
                    for (DynamicObject row : oldManner) {
                        DynamicObject newEntryRow = newManner.addNew();
                        newEntryRow.set("yrpg_link_man", row.get("link_man"));
                        newEntryRow.set("yrpg_contact_address", row.get("contact_address"));
                        newEntryRow.set("yrpg_contact_phone", row.get("contact_phone"));
                        newEntryRow.set("yrpg_contact_telephone", row.get("contact_telephone"));
                        newEntryRow.set("yrpg_contact_email", row.get("contact_email"));
                    }
                }
                basicData.set("yrpg_sourcebillid", data.getString("id"));
                basicData.set("yrpg_soumodifytime", data.get("modifytime"));
                SaveServiceHelper.save(new DynamicObject[]{basicData});
            } else {
                //新增
                DynamicObject newData = BusinessDataServiceHelper.newDynamicObject(basicEntity.getSynKey());
                for (String mapKey : fieldset) {
                    String filedKey;
                    if (mapKey.contains("yrpg_")) {
                        int index = mapKey.indexOf("yrpg_") + 5;
                        filedKey = mapKey.substring(index);
                    } else {
                        filedKey = mapKey;
                    }
                    if (billKey.containsKey(filedKey)) {
                        filedKey = billKey.get(filedKey);
                    }
                    if (sourceAllKey.contains(filedKey)) {
                        if (localName.equals("医院") && !Arrays.asList(CAMPUS_KEY).contains(filedKey)) {
                            newData.set(mapKey, data.get(filedKey));
                            if (largeTextKey.contains(mapKey)) {
                                newData.set(mapKey + "_tag", data.get(filedKey + "_tag"));
                            }
                        } else {
                            newData.set(mapKey, data.get(filedKey));
                            if (largeTextKey.contains(mapKey)) {
                                newData.set(mapKey + "_tag", data.get(filedKey + "_tag"));
                            }
                        }
                    }
                }
                if (localName.equals("医生")) {
                    newData.set(statusKey, "A");
                    newData.set("yrpg_datasource", "B");
                } else {
                    newData.set(statusKey, "C");
                    if (!localName.equals("客户管理") && !localName.equals("人员档案") && !localName.equals("终端表(正式)") && !localName.equals("医院")) {
                        newData.set("auditor", RequestContext.get().getCurrUserId());
                        newData.set("auditdate", new Date());
                    }
                    if (localName.equals("医院")) {
                        DynamicObjectCollection newTrdHospital = newData.getDynamicObjectCollection("yrpg_rm_med_trd_hospital");
                        DynamicObjectCollection hospitalEntry = data.getDynamicObjectCollection("rm_med_trd_hospital_detai");
                        for (DynamicObject row : hospitalEntry) {
                            DynamicObject newEntryRow = newTrdHospital.addNew();
                            newEntryRow.set("yrpg_branchesid", row.get("branchesid"));
                            newEntryRow.set("yrpg_district_address", row.get("district_address"));
                            newEntryRow.set("yrpg_district", row.get("district"));
                            newEntryRow.set("yrpg_area", row.get("area"));
                            newEntryRow.set("yrpg_longitude", row.get("longitude"));
                            newEntryRow.set("yrpg_latitude", row.get("latitude"));
                            newEntryRow.set("yrpg_master_number", row.get("master_number"));
                            newEntryRow.set("yrpg_area_code", row.get("area_code"));
                            newEntryRow.set("yrpg_is_active", row.get("is_active"));
                        }
                    } else if (localName.equals("客户管理")) {
                        newData.set("yrpg_auditor", RequestContext.get().getCurrUserId());
                        newData.set("yrpg_auditdate", new Date());
                        //来源系统
                        DynamicObjectCollection oldSrcSys = data.getDynamicObjectCollection("customer_src_sys");
                        DynamicObjectCollection newSrcSys = newData.getDynamicObjectCollection("yrpg_customer_src_sys");
                        for (DynamicObject row : oldSrcSys) {
                            DynamicObject newEntryRow = newSrcSys.addNew();
                            newEntryRow.set("yrpg_src_sys_code", row.get("src_sys_code"));
                        }
                        //客户所在地址
                        DynamicObjectCollection oldAddress = data.getDynamicObjectCollection("customer_address");
                        DynamicObjectCollection newAddress = newData.getDynamicObjectCollection("yrpg_customer_address");
                        for (DynamicObject row : oldAddress) {
                            DynamicObject newEntryRow = newAddress.addNew();
                            newEntryRow.set("yrpg_address", row.get("address"));
                            newEntryRow.set("yrpg_default_address", row.get("default_address"));
                        }
                        //联系方式
                        DynamicObjectCollection oldManner = data.getDynamicObjectCollection("contact_manner");
                        DynamicObjectCollection newManner = newData.getDynamicObjectCollection("yrpg_contact_manner");
                        for (DynamicObject row : oldManner) {
                            DynamicObject newEntryRow = newManner.addNew();
                            newEntryRow.set("yrpg_link_man", row.get("link_man"));
                            newEntryRow.set("yrpg_contact_address", row.get("contact_address"));
                            newEntryRow.set("yrpg_contact_phone", row.get("contact_phone"));
                            newEntryRow.set("yrpg_contact_telephone", row.get("contact_telephone"));
                            newEntryRow.set("yrpg_contact_email", row.get("contact_email"));
                        }
                    } else if (localName.equals("人员档案")) {
                        newData.set("yrpg_auditor", RequestContext.get().getCurrUserId());
                        newData.set("yrpg_auditdate", new Date());
                        newData.set("yrpg_sourcefid", data.getLong("id"));
                        sourceBillId.add(data.getString("id"));
                    }
                }
                newData.set("yrpg_sourcebillid", data.getString("id"));
                newData.set("creator", data.get("creator"));
                newData.set("createtime", data.get("createtime"));
                newData.set("yrpg_soumodifytime", data.get("modifytime"));
                SaveServiceHelper.save(new DynamicObject[]{newData});
            }
        }
        return sourceBillId;
    }

    private static Set<String> getFieldKey(String number) {
        MainEntityType dataEntityType = MetadataServiceHelper.getDataEntityType(number);
        Map<String, IDataEntityProperty> allFields = dataEntityType.getAllFields();
        HashMap<String, String> map = new HashMap<>(allFields.size());
        for (String item : allFields.keySet()) {
            IDataEntityProperty iDataEntityProperty = allFields.get(item);
            String name = iDataEntityProperty.getName();
            String localeValue = iDataEntityProperty.getDisplayName().getLocaleValue();
            map.put(name, localeValue);
        }

        map.remove("creator");
        map.remove("modifier");
        map.remove("createtime");
        map.remove("modifytime");
        Set<String> fieldset = map.keySet();
        return fieldset;
    }

    /**
     * 大文本未处理
     *
     * @param localName
     * @return
     */
    private static Map<String, String> getBillKey(String localName) {
        Map<String, String> map = new HashMap<>();
        switch (localName) {
            case "终端表(正式)":
                map.put("secondary_category_n", "secondary_category_name");
                map.put("storeheadquartersnam", "storeheadquartersname");
                map.put("tertiary_category_na", "tertiary_category_name");
                map.put("ismedicaldoublechann", "ismedicaldoublechannel");
                map.put("storeheadquarterscod", "storeheadquarterscode");
                map.put("primary_category_nam", "primary_category_name");
                break;
            case "医生":
                map.put("government_organizat", "government_organization");
                map.put("primary_practice_hos", "primary_practice_hospital");
                map.put("standard_department_", "standard_department_name");
                break;
            case "医院":
                map.put("organization_real_ca", "organization_real_capital");
                map.put("operation_started_da", "operation_started_date");
                map.put("secondary_category_n", "secondary_category_name");
                map.put("primary_category_nam", "primary_category_name");
                map.put("tertiary_category_na", "tertiary_category_name");
                map.put("government_organizat", "government_organization");
                map.put("unified_social_credi", "unified_social_credit_cod");
                map.put("business_license_num", "business_license_number");
                map.put("tax_registration_num", "tax_registration_number");
                map.put("organization_reg_cap", "organization_reg_capital");
                break;
            case "客户管理":
                map.put("is_private_medicalgr", "is_private_medicalgroup");
                map.put("business_distinct_ty", "business_distinct_type");
                map.put("general_hospital_nam", "general_hospital_name");
                map.put("general_hospital_num", "general_hospital_number");
                map.put("flow_customer_type_i", "flow_customer_type_id");
                break;
            case "人员档案":
                map.put("firstsalesteam1", "firstsalesteam");
                map.put("secondsalesteam1", "secondsalesteam");
                break;
            default:
                break;
        }
        return map;
    }

    private List<String> getLargeTextKey(String localName) {
        List<String> list = new ArrayList<>();
        switch (localName) {
            case "终端表(正式)":
                list.add("yrpg_previous_names");
                list.add("yrpg_organization_scope");
                list.add("yrpg_key_departments");
                break;
            case "医生":
                list.add("yrpg_primary_practice_hos");
                list.add("yrpg_multiple_sites");
                list.add("yrpg_personal_profile");
                list.add("yrpg_expertise_area");
                break;
            case "医院":
                list.add("yrpg_previous_names");
                list.add("yrpg_organization_scope");
                list.add("yrpg_key_departments");
                list.add("yrpg_medical_projects");
                break;
            default:
                break;
        }
        return list;
    }
}
