package yrpg.bamp.basedata.webapi;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.openapi.api.result.ApiSaveItemData;
import kd.bos.openapi.api.result.ApiSaveResult;
import kd.bos.openapi.api.result.ApiSaveRowErrorData;
import kd.bos.openapi.api.result.BaseFilterItemData;
import kd.bos.openapi.api.result.BaseFilterResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.custom.annotation.ApiResponseBody;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.openapi.common.result.OpenApiResult;
import kd.bos.openapi.common.util.OpenApiSdkUtil;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import yrpg.bamp.basedata.common.ApiReqModel;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author: zhiwei xing
 * @description 归集码接口
 * @date: 2024/5/18 13:58
 */

@ApiController(value="GoodsCollectionApi",desc = "归集码接口统一url入口")
public class GoodsCollectionApiController {

    private final static String ENTITYNAME = "yrpg_goods_collection" ;

    @ApiPostMapping(value="batchSyncGoodsCollection")
    public CustomApiResult<@ApiResponseBody("Body") Map<String,Object>> goodsCollApiHandler(@Valid @ApiParam("入口参数") ApiReqModel reqModel) {
        String reqData=reqModel.getData();
        JSONArray reqList = JSONObject.parseArray(reqData);

        String entityCode = null;//主数据分发数据所属的数据模型对应实体编码
        //获取系统现有归集码数据,并且将系统中的启用禁用状态、主数据编码分别放入两个map,分组判断时从map中获取数据，用于调用不同的接口（保存、启用、禁用）
        Map<String,String> enableStatusMap = new HashMap<String,String>();
//        Map<String,String> entityCodeMap = new HashMap<String,String>();
        Map<String,String> numberMdmCodeMap = new HashMap<String,String>(); //编码和主数据编码对应关系
        DynamicObjectCollection systemGoodsCollList = QueryServiceHelper.query(ENTITYNAME, "number,yrpg_mdm_code,enable", new QFilter[]{});
        for(DynamicObject sysGoodsCollData : systemGoodsCollList){
            enableStatusMap.put(sysGoodsCollData.getString("yrpg_mdm_code"),sysGoodsCollData.getString("enable"));
        }

        JSONArray saveList = new JSONArray();       //只需要进行save操作的数据
        JSONArray enableList = new JSONArray();     //启用数据,先进行启用操作，再save
        JSONArray disableList = new JSONArray();    //禁用数据，直接禁用操作，不需要save
        JSONArray notTodoList = new JSONArray();       //不需要进行任何操作的数据，返回时直接认为操作成功
        JSONArray noNumberList = new JSONArray();       //传入的数据没有number，直接做失败处理
        List<String> disableIds = new ArrayList<String>();
        List<String> enableIds = new ArrayList<String>();

        //数据分组，根据系统中是否已存在、启用、禁用状态进行分组
        for(int i=0; i<reqList.size(); i++){
            JSONObject reqJson = reqList.getJSONObject(i);
            if(StringUtils.isEmpty(entityCode)){
                entityCode = reqJson.getString("entityCode");
            }
            String yrpg_mdm_code = reqJson.getString("yrpg_mdm_code");//主数据编码
            String number = reqJson.getString("number");//归集码编码，这里要求调用方传入的编码和主数据编码字段值一致
            numberMdmCodeMap.put(number,yrpg_mdm_code);//归集码编码和主数据编码映射关系
            String enableStatus = reqJson.getString("enable");

            //--------------传入number为空的数据-------------
            if(StringUtils.isBlank(yrpg_mdm_code) || StringUtils.isEmpty(yrpg_mdm_code)){
                noNumberList.add(reqJson);
            } else if(enableStatusMap.get(yrpg_mdm_code) == null){
                //-------------------系统中暂不存在的数据------------------------
                if("0".equals(enableStatus)){
                    //不存在，且需要禁用的，不做处理，放入notTodoList
                    notTodoList.add(reqJson);
                }else{
                    //不存在，且需要启用的，直接save，默认就是开启状态，放入saveList
                    saveList.add(reqJson);
                }
            }else{
                //-------------------系统中已存在的数据---------------------
                if("1".equals(enableStatus) && "1".equals(enableStatusMap.get(yrpg_mdm_code))){
                    //启用，系统中已经是启用状态的,只需要save,放入saveList
                    saveList.add(reqJson);
                } else if ("1".equals(enableStatus) && "0".equals(enableStatusMap.get(yrpg_mdm_code))) {
                    //启用，系统中是禁用状态的,放入enableList
                    enableList.add(reqJson);
                    enableIds.add(yrpg_mdm_code);
                } else if ("0".equals(enableStatus) && "0".equals(enableStatusMap.get(yrpg_mdm_code))) {
                    //禁用，系统中已经是禁用状态的，不需要任何操作，放入notTodoList
                    notTodoList.add(reqJson);
                }else if("0".equals(enableStatus) && "1".equals(enableStatusMap.get(yrpg_mdm_code))){
                    //禁用，系统中是启用状态的，放入disableList
                    disableList.add(reqJson);
                    disableIds.add(yrpg_mdm_code);
                }else{
                    //不满足条件的统一不做任何处理
                    notTodoList.add(reqJson);
                }
            }
            //-------------------分组判断结束------------------------------
        }

        List successData = new ArrayList();
        List<JSONObject> mdMappingsList = new ArrayList<JSONObject>();
        //save数据处理
        if(saveList.size() > 0){
            JSONObject saveJson = new JSONObject();
            saveJson.put("data",saveList);
            OpenApiResult saveReult = OpenApiSdkUtil.invoke("/v2/yrpg/rm_mkt_md_manage/yrpg_goods_collection/goods_collection_save", saveJson, new HashMap<>());
            formatRetValue(mdMappingsList,saveReult,entityCode,numberMdmCodeMap,"save",successData);
        }

        //禁用数据处理
        if(disableList.size() > 0){
            JSONObject disableJson = new JSONObject();
            JSONObject ids = new JSONObject();
            ids.put("yrpg_mdm_code",disableIds);
            disableJson.put("data",ids);
            OpenApiResult disableReult = OpenApiSdkUtil.invoke("/v2/yrpg/rm_mkt_md_manage/yrpg_goods_collection/goods_collection_disable", disableJson, new HashMap<>());
            formatRetValue(mdMappingsList,disableReult,entityCode,numberMdmCodeMap,"disable",successData);
        }

        //启用数据处理
        if(enableList.size() > 0) {
            JSONObject enableJson = new JSONObject();
            JSONObject ids = new JSONObject();
            ids.put("yrpg_mdm_code",enableIds);
            enableJson.put("data",ids);
            //调用启用接口
            OpenApiSdkUtil.invoke("/v2/yrpg/rm_mkt_md_manage/yrpg_goods_collection/goods_collection_enable", enableJson, new HashMap<>());
            //反审核
//            OpenApiSdkUtil.invoke("/v2/yrpg/rm_mkt_md_manage/yrpg_goods_collection/goods_collection_unaudit", enableJson, new HashMap<>());
            //调用save接口
            enableJson.put("data",enableList);
            OpenApiResult saveReult = OpenApiSdkUtil.invoke("/v2/yrpg/rm_mkt_md_manage/yrpg_goods_collection/goods_collection_save", enableJson, new HashMap<>());
            formatRetValue(mdMappingsList, saveReult, entityCode,numberMdmCodeMap, "save",successData);
        }

        //不需要操作的数据直接封装成功返回
        if(notTodoList.size()>0){
            for(int i=0;i<notTodoList.size();i++){
                JSONObject reqJson = notTodoList.getJSONObject(i);
                JSONObject json = new JSONObject();
                json.put("mdmCode",reqJson.getString("yrpg_mdm_code"));
                json.put("entityCode",reqJson.getString("entityCode"));
                json.put("busiDataId","");
                json.put("message","该条主数据消费成功！");
                json.put("success",true);
                mdMappingsList.add(json);
                successData.add(reqJson.getString("yrpg_mdm_code"));
            }
        }

        Map<String,Object> result = new HashMap<String,Object>();
        if(successData.size()==0){
            result.put("success",false);
            result.put("message", "消费系统消费这批数据全部失败！");
        }else if(successData.size()==reqList.size()){
            result.put("success",true);
            result.put("message", "消费系统消费这批数据成功！");
        }else{
            result.put("success",false);
            result.put("message", "消费系统消费这批数据部分成功！");
        }
        result.put("mdMappings",mdMappingsList);
        return CustomApiResult.success(result);
    }

    //按照主数据系统报文格式封装返回值
    private void formatRetValue(List<JSONObject> mdMappingsList,OpenApiResult result,String entityCode,Map<String,String> numberMdmCodeMap,String operateType,List successData){
        if("save".equals(operateType)) {
            ApiSaveResult data = (ApiSaveResult) result.getData();
            List<ApiSaveItemData> saveItemList = data.getResult();
            for (ApiSaveItemData item : saveItemList) {
                String mdmCode = item.getKeys().get("yrpg_mdm_code");
                if (!StringUtils.isEmpty(mdmCode)) {
                    JSONObject jsonNew = new JSONObject();
                    jsonNew.put("mdmCode", mdmCode);
                    jsonNew.put("entityCode", entityCode);
                    jsonNew.put("success", item.isBillStatus());
                    if (item.isBillStatus()) {
                        successData.add(mdmCode);
                        jsonNew.put("busiDataId",item.getId());
                        jsonNew.put("message", "该条主数据消费成功！");
                        mdMappingsList.add(jsonNew);
                    } else {
                        StringBuffer errmsg = new StringBuffer();
                        for (ApiSaveRowErrorData errer : item.getErrors()) {
                            //明细错误拼接
                            Set<String> rowMsg = errer.getRowMsg();
                            for (String err : rowMsg) {
                                errmsg.append(err);
                            }
                        }
                        jsonNew.put("busiDataId", null);
                        jsonNew.put("message", errmsg.toString());
                        mdMappingsList.add(jsonNew);
                    }
                }
            }
        }else{
            BaseFilterResult data = (BaseFilterResult)result.getData();
            List<BaseFilterItemData> itemList = data.getResult();
            for(BaseFilterItemData item : itemList){
                JSONObject json = new JSONObject();
                String number = item.getNumber();
                json.put("mdmCode",numberMdmCodeMap.get(number));
                json.put("entityCode",entityCode);
                json.put("busiDataId",item.getId());
                json.put("success",item.isBillStatus());
                if(item.isBillStatus()){
                    successData.add(numberMdmCodeMap.get(number));
                    json.put("message","该条主数据消费成功！");
                }else{
                    StringBuffer errmsg = new StringBuffer();
                    List<String> errors = item.getErrors();
                    for(String err : errors){
                        errmsg.append(err);
                    }
                    json.put("message",errmsg.toString());
                }
                mdMappingsList.add(json);
            }

        }
    }

}
